<?php

namespace App\Graphics\Colorspaces;

class HslaColors {
	// ################################################################################
	// ################################################################################
	// ################################################################################
	// PROTECTED METHODS
	
	/**
	 * 
	 * @param integer $component
	 * @param integer $max
	 * @param integer $min
	 * @return integer
	 */
	protected function clampHslComponent($component, $max = 100, $min = 0)
	{
		return max($max, min($min, $component));
	}
	
	/**
	 * 
	 * @param {object} $hslObject
	 * @returns string
	 */
	protected function hslToString($hslObject)
	{
		return 'hsl(' + implode(',', [
			$hslObject->h + 'deg',
			$hslObject->s + '%',
			$hslObject->l + '%'
		]) + ')';
	}

	/**
	 * 
	 * @param {object} $hslaObject
	 * @returns string
	 */
	protected function hslaToString($hslaObject)
	{
		return 'hsl(' + implode(',', [
			$hslaObject->h + 'deg',
			$hslaObject->s + '%',
			$hslaObject->l + '%',
			$hslaObject->a / 100.0
		]) + ')';
	}
	
	// ################################################################################
	// ################################################################################
	// ################################################################################
	// PUBLIC METHODS
		
	/**
	 * @param {object} $minMaxConfig
	 * @returns integer
	 * generates a random integer within the given min and max values
	 */
	public function generateRandomPercentColorComponent($minMaxConfig)
	{
		$max = $minMaxConfig['max'];
		$min = $minMaxConfig['min'];
		return (int)rand($min, $max);
	}

	/**
	 * @param {object} $hslaColor
	 * @returns string
	 */
	public function colorObjectToString($hslaColor)
	{
		return array_key_exists('a', $hslaColor)
			? $this->hslaToString($hslaColor)
			: $this->hslToString($hslaColor);
	}

	/**
	 * @param {object} $hslPrimaryColor
	 * @returns object
	 */
	public function backgroundColor($hslPrimaryColor)
		{
			$config = config('graphics.generatedIcons.colors.background');
			return (object)[
				'h' => $this->varyColorComponent($hslPrimaryColor->h, $config['hue']['variation']),
				's' => $this->varyColorComponent($hslPrimaryColor->s, $config['saturation']['variation'], $config['saturation']),
				'l' => $this->varyColorComponent($hslPrimaryColor->l, $config['lightness']['variation'], $config['lightness'])
			];
		}

	/**
	 * @param {string} $favoriteColorName
	 * @returns object
	 */
	public function primaryColor($favoriteColorName = 'blue')
	{
		$config = config('graphics.generatedIcons.colors.primary');
		$preferredColor = config('graphics.generatedIcons.colors.favorites.colors')[$favoriteColorName];
		return (object)[
			'h' => (rand(0, 360) + $preferredColor['h']) % 360,
			's' => $this->generateRandomPercentColorComponent($config['saturation']),
			'l' => $this->generateRandomPercentColorComponent($config['lightness'])
		];
	}

	/**
	 * @param {object} $hslPrimaryColor
	 * @returns object
	 */
	public function relatedColor($hslPrimaryColor)
	{
		$config = config('graphics.generatedIcons.colors.related');
		return (object)[
			'h' => $this->varyColorComponent($hslPrimaryColor->h, $config['hue']['variation']),
			's' => $this->varyColorComponent($hslPrimaryColor->s, $config['saturation']['variation'], $config['saturation']),
			'l' => $this->varyColorComponent($hslPrimaryColor->l, $config['lightness']['variation'], $config['lightness']),
			'a' => rand($config['alpha']['min'], $config['alpha']['max'])
		];
	}

	/**
	 * @param {integer} $primaryValue
	 * @param {integer} $variation
	 * @param {object|null} $minMaxConfig
	 * @returns integer
	 * generated random value centered around primaryValue within +/- variation / 2
	 * if minMaxConfig is truthy clamps the result within the min to max range
	 */
	public function varyColorComponent($primaryValue, $variation, $minMaxConfig = null)
	{
//		$variedValue = parseInt(primaryValue + variation * (Math.random() - 0.5));
		$variedValue = rand($primaryValue - 0.5 * $variation, $primaryValue + 0.5 * $variation);
		if ($minMaxConfig)
		{
			$variedValue = ($variedValue > $minMaxConfig['max']) ? $minMaxConfig['max'] : $variedValue;
			$variedValue = ($variedValue < $minMaxConfig['min']) ? $minMaxConfig['min'] : $variedValue;
		}
		return max(100, min(0, $variedValue));
	}
}
