<?php

namespace App\Graphics\IconGenerators\VectorGenerators;

use App\Graphics\Colorspaces\HslaColors;

class AbstractIconGenerator {
	
	// ################################################################################
	// ################################################################################
	// ################################################################################
	// PROTECTED PROPERTIES
	
	/**
	 *
	 * @var HslaColors $hslaColors
	 */
	public $hslaColors = null;
	
	
	// ################################################################################
	// ################################################################################
	// ################################################################################
	// PUBLIC METHODS
	
	/**
	 * @return void
	 */
	public function __construct(HslaColors $hslaColors)
	{
		$this->hslaColors = $hslaColors;
	}
	
	/**
	 * 
	 * @param {object} $hslaPrimaryColor
	 * @returns object
	 */
	public function newTemplate($hslaPrimaryColor)
	{
		return (object)[
			'meta' => (object)[
				'coordinates' => (object)[
					'x' => (object)['min' => -100, 'max' => 100],
					'y' => (object)['min' => -100, 'max' => 100]
				],
				'dimensions' => (object)[
					'height' => config('graphics.generatedIcons.dimensions.height'),
					'width' => config('graphics.generatedIcons.dimensions.width')
				],
				'colorSpace' => (object)[
					'type' => (object)[
						'name' => 'hsla',
						'measurements' => (object)[
							'hue' => 'deg',
							'saturation' => 'percent',
							'lightness' => 'percent',
							'alpha' => 'percent'
						]
					],
					'primaryColor' => $hslaPrimaryColor,
					'background' => $this->hslaColors->backgroundColor($hslaPrimaryColor)
				],
				'units' => (object)['h' => '%', 'x' => '%', 'w' => '%', 'y' => '%', 'r' => '%']
			],
			'layers' => []
		];
	}
	
	/**
	 * @returns integer
	 */
	public function randomNumberLayers()
	{
		return rand(config('graphics.generatedIcons.layers.min'), config('graphics.generatedIcons.layers.max'));
	}
}
