<?php

namespace App\Graphics\IconGenerators\VectorGenerators;

use App\Graphics\Colorspaces\HslaColors;

class AbstractIconLayerGenerator {
	// ################################################################################
	// ################################################################################
	// ################################################################################
	// PROTECTED PROPERTIES
	
	/**
	 *
	 * @var array $availableShapes
	 */
	protected $availableShapes = [];
	
	/**
	 *
	 * @var HslaColors $hslaColorUtilities
	 */
	protected $hslaColorUtilities = null;
	
	/**
	 *
	 * @var object $primaryColor
	 */
	protected $primaryColor = null;
	
	/**
	 *
	 * @var array $shapeGenerators
	 */
	protected $shapeGenerators = [];
	 
	// ################################################################################
	// ################################################################################
	// ################################################################################
	// PROTECTED METHODS

	/**
	 * @return boolean
	 */
	protected function coinToss()
	{
		return rand(0,1);
	}
	
	/**
	 * 
	 * @param float $degrees
	 * @return float
	 */
	protected function degreesToRadians($degrees)
	{
		return $degrees / 180.0 * pi();
	}

	/**
	 * @return float 0 - 100.0
	 */
	protected function randomPosition(){
		return 0.1 * rand(0, 1000);
	}
	
	// ################################################################################
	// ################################################################################
	// ################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param HslaColors $hslaColorUtilities
	 * @return void
	 */
	public function __construct(HslaColors $hslaColorUtilities)
	{
		$this->hslaColorUtilities = $hslaColorUtilities;
		
		/**
		 * @param array $config
		 * @return object
		 */
		$this->shapeGenerators['circle'] = function ($config)
		{
			$angleStart = rand(0, 360);
			return (object)[
				'x' => $this->randomPosition(),
				'y' => $this->randomPosition(),
				'r' => 0.1 * rand(10.0 * $config['radius']['min'], 10.0 * $config['radius']['max']),
				'angleStart' => $this->degreesToRadians($angleStart),
				'angleEnd' => $this->degreesToRadians($angleStart + rand(0, 360))
			];
		};

		/**
		 * @param array $config
		 * @return object
		 */
		$this->shapeGenerators['polygonRandom'] = function ($config)
		{
			$numberVertices = rand($config['vertices']['min'], $config['vertices']['max']);
			$vertices = [];
			for ($i = 0; $i < $numberVertices; $i++) {
				$vertices[] = (object)[
					'x' => $this->randomPosition(),
					'y' => $this->randomPosition()
				];
			}
			return $vertices;
		};

//		polygonRegular: function (config){
//
//		},

		/**
		 * @param array $config
		 * @return object
		 */
		$this->shapeGenerators['rectangle'] = function ($config)
		{
			return (object)[
				'x' => $this->randomPosition(),
				'y' => $this->randomPosition(),
				'h' => 0.1 * rand(0, 10.0 * $config['height']),
				'w' => 0.1 * rand(0, 10.0 * $config['width'])
			];
		};
		
		$this->availableShapes = array_keys($this->shapeGenerators);
	}

	/**
	 * 
	 * @returns object
	 */
	public function generate()
	{
		$transforms = (object)[];
		$borderMin = config('graphics.generatedIcons.borders.min');
		$borderMax = config('graphics.generatedIcons.borders.max');
		$styles = (object)[];
		$styles->line = (object)[
			'color' => $this->hslaColorUtilities->relatedColor($this->primaryColor),
			'widthPercent' => 0.1 * rand(10.0 * $borderMin, 10.0 * $borderMax)
		];
		// if ($this->coinToss()) {
			$styles->fill = $this->hslaColorUtilities->relatedColor($this->primaryColor);
		// }
		$shape = (object)[];
		$shapeType = $this->availableShapes[rand(0, count($this->availableShapes) - 1)];
		$shape->type = $shapeType;
		$shape->data = $this->shapeGenerators[$shapeType](config("graphics.generatedIcons.shapes.$shapeType"));
		return (object)[
			'transforms' => $transforms,
			'styles' => $styles,
			'shape' => $shape
		];
	}
	
	/**
	 * 
	 * @param object $hslColor
	 */
	public function setPrimaryColor($hslColor)
	{
		$this->primaryColor = $hslColor;
	}
}
