<?php

namespace App\Http\Controllers\Api\Channels;

use App\Http\Requests\Channels\ChannelDetailsRequest as Request;

use App\Enums\Channels\ChannelRolesEnum;
use App\Http\Controllers\Controller;
use App\Jobs\Channels\AddMember as AddChannelMember;
use App\Jobs\Channels\BroadcastNewChannel;
use App\Jobs\Channels\CreateChannelWithAccountAssociation;
use App\Jobs\Invitations\SendChannelInvitations;
use App\Jobs\UnlockPivotEncryptedInfo\AccountOwner as UnlockAccountOwner;
//use App\Http\Requests\Channels\ChannelDetailsRequest as Request;
use App\Traits\Authentication\AuthenticationHelperTraits;
use App\Traits\Conversions\ConversionHelperTraits;
use App\Traits\Strings\Encryption\EncryptionHelperTraits;
use App\Transports\Channels\CreationInformation as ChannelCreationInformation;
use App\Transports\Information\CommonRecordInfoReturn;
use App\Transports\Information\Description;
use App\Transports\Users\Details as UserDetails;

class CreateChannelController extends Controller
{
	use ConversionHelperTraits, EncryptionHelperTraits,
		AuthenticationHelperTraits;
	
    public function create(Request $request)
	{
		$userDataKeys = $request->getUserDataKeys();
		$user = $request->user();
		
		$accountInfo = UnlockAccountOwner::dispatchNow(
			$userDataKeys,
			$user->toArray(),
			$request->route('accountHash')
		);
		$channelDescription = new Description(
			$request->input('title'),
			$request->input('description'),
			($request->input('encryptedIcon')) ? true : false
		);
		$channel = CreateChannelWithAccountAssociation::dispatchNow(
			new ChannelCreationInformation(
				$request->input('encryptionKey'),
				$channelDescription,
				$request->input('encryptedIcon'),
				$request->input('searchable'),
				new Description(
					$request->input('searchableTitle'),
					$request->input('searchableDescription')
				),
				$request->input('contentExpiresDays'),
				$request->input('typeId')
			),
			$accountInfo->hashes->lookup,
			$accountInfo->keys->keyPair->publicKey
		);
		
		$decryptedUserInfo = $this->decryptByStringKey(
			$user->encrypted_info,
			$userDataKeys->userEncryptionKey,
			true
		);

		AddChannelMember::dispatch(
			new UserDetails(
				$decryptedUserInfo->email,
				$decryptedUserInfo->nickName,
				$decryptedUserInfo->icon,
				$decryptedUserInfo->iconFallback
			),
			$decryptedUserInfo->hashes->lookup,
			$decryptedUserInfo->keys->keyPair->publicKey,
			$channelDescription,
			$channel->hashes,
			$channel->keys,
			$channel->id,
			ChannelRolesEnum::administrator,
			$request->input('contentExpiresDays')
		);
		
		BroadcastNewChannel::dispatch(
			$this->convertEmailsToHash($request->input('invitations')),
			$userDataKeys->deviceHash
		);
		
		SendChannelInvitations::dispatch($channel, $request->input('invitations'));

		return response()
			->json(new CommonRecordInfoReturn(
				$channel->hashes->lookup,
				$channel->privateDescription,
				$channel->id
			));
	}
}
