<?php

namespace App\Http\Controllers\Api\Content;

use App\Enums\Encryption\StringEncryptionAlgorithmsEnum;
use App\Http\Requests\Authentication\AuthenticatedRequest as Request;
use App\Jobs\Content\AuthorInfoFromContent;
use App\Jobs\Content\LoadContentSince;
use App\Jobs\Content\LoadDiscussion;
use App\Jobs\Content\LoadEncryptedData;
use App\Jobs\Content\LoadRecentContent;
use App\Jobs\StringKeys\ChannelStringKeys;
use App\Traits\Conversions\ConversionHelperTraits;
use App\Traits\Strings\Encryption\EncryptionHelperTraits;
use App\Transports\Information\ID;

//use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class LoadContentController extends Controller
{
	use ConversionHelperTraits, EncryptionHelperTraits;
	
	/**
	 * 
	 * @param Request $request
	 * @return string
	 */
	protected function getChannelKeys(Request $request)
	{
		$user = $request->user();
		$keys = ChannelStringKeys::dispatchNow(
				$request->getUserDataKeys(),
				$user->toArray(),
				$request->route('channelHash')
		);
		// <<<< [TODO]
//		return ($keys)
//			? $this->encryptByStringKey(
//				$keys,
//				$user->device_server_crypto_key,
//				timestamp(),
//				false,
//				StringEncryptionAlgorithmsEnum::MAX_COMPATABILITY
//			)
//			: '';
		return ($keys) ? $keys : '';
	}
	
	
	/**
	 * 
	 * @param Request $request
	 * @return string
	 */
	public function discussion(Request $request)
	{
		$content = LoadDiscussion::dispatchNow(
			$request->route('channelHash'),
			$request->route('timestamp')
		);
		return response()
			->json((object)compact('content'));
	}
	
	
	/**
	 * 
	 * @param Request $request
	 * @return string
	 */
	public function encryptedData(Request $request)
	{
		return response()
			->json(LoadEncryptedData::dispatchNow(
				new ID($request->route('id'), $request->route('cluster_id'))
			));
	}
	
	
	/**
	 * 
	 * @param Request $request
	 * @return string
	 */
    public function recent(Request $request)
	{
		$userDataKeys = $request->getUserDataKeys();
		$userRecord = $request->user()->toArray();
		$channelHash = $request->route('channelHash');
		
		$content = LoadRecentContent::dispatchNow(
			$channelHash,
			(int)config('content.recordsPerPage') * (int)$request->route('page')
		);
//		$keys = $this->getChannelKeys($request);
//		return response()
//			->json((object)compact('content', 'keys'));
		return response()
			->json((object)compact('content'));
	}
	
	
	/**
	 * 
	 * @param Request $request
	 * @return string
	 */
	public function since(Request $request)
	{
		$userDataKeys = $request->getUserDataKeys();
		$userRecord = $request->user()->toArray();
		$channelHash = $request->route('channelHash');
		
		$content = LoadContentSince::dispatchNow(
			$channelHash,
			$request->input('timestamp'),
			(int)config('content.recordsPerPage') * (int)$request->route('page')
		);
//		$keys = $this->getChannelKeys($request);
//		return response()
//			->json((object)compact('content', 'keys'));
		return response()
			->json((object)compact('content'));
	}
}
