<?php

namespace App\Http\Controllers\Api\Users;

use App\Http\Controllers\Controller;
use App\Http\Requests\Users\SignUpRequest as Request;
use App\Jobs\AccountsAndUsers\CreateAccountAndUser;
use App\Transports\Accounts\CreationInformation as AccountCreationInformation;
use App\Transports\Authentication\AuthenticationCredentials;
use App\Transports\Encryption\NewDeviceKeys;
use App\Transports\Information\Description;
use App\Transports\Information\DeviceInformation;
use App\Transports\Users\UserAuthCredentials;

class SignUpController extends Controller
{
	public function signUp(Request $request)
	{
		$accountAndUser = CreateAccountAndUser::dispatchNow(
			new AccountCreationInformation(
				new Description(
					trans('accounts.defaultTitle'),
					trans('accounts.defaultDescription')
				)
			),
			new AuthenticationCredentials(
				new UserAuthCredentials(
					$request->input('email'),
					$request->input('password'),
					$request->input('phrase')
				),
				new DeviceInformation(
					$request->server('REMOTE_ADDR'),
					$request->server('HTTP_USER_AGENT')
				)
			)
		);
		$encryptionKeys = new NewDeviceKeys(
			$accountAndUser->user->userEncryptionKey,
			$accountAndUser->user->hashes->lookup,
			$accountAndUser->bearerToken->deviceHash,
			$accountAndUser->user->keys->deviceServerCryptoKey
		);
		return response()
			->json(
				$encryptionKeys,
				self::HTTP_STATUS_SUCCESS,
				['Authorization' => 'Bearer ' . $accountAndUser->bearerToken->bearerToken]
			);
	}
}