<?php

namespace App\Http\Middleware;

use App\Http\Middleware\MiddlewareBase;
use App\Jobs\Authentication\Memberships\IsChannelMember as IsChannelMemberJob;
use App\Jobs\Logs\LogFailedValidationRouteAccess;
use App\Traits\Strings\Encryption\UnlockEncryptedRequestKeysTraits;

use Closure;

class IsChannelMember extends MiddlewareBase
{
	use UnlockEncryptedRequestKeysTraits;
	
    /**
     * Handle an incoming request.
     *
     * @param  AuthenticatedRequest $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
		$keys = $this->unlockRequestHeaderKeys($request);
		if (IsChannelMemberJob::dispatchNow(
			$keys->userLookupHash,
			$request->route('channelHash')
		))
		{
			return $next($request);
		}
		LogFailedValidationRouteAccess::dispatch(
			$request->route()->uri,
			self::class,
			$request->server('REMOTE_ADDR'),
			($request->user()) ? $request->user()->device_hash : ''
		);
		return $this->forbidden();
    }
}
