<?php

namespace App\Http\Middleware;

use App\Http\Middleware\MiddlewareBase;
use App\Interfaces\Constants\DeviceConstants;
use App\Jobs\Logs\LogFailedValidationRouteAccess;
use App\Traits\Strings\Encryption\UnlockEncryptedRequestKeysTraits;

use Closure;

class IsKnownDevice extends MiddlewareBase
{
	use UnlockEncryptedRequestKeysTraits;
	
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
		if ($request->user()->device_status == DeviceConstants::DEVICE_STATUS_ALLOW)
		{
			return $next($request);
		}
		LogFailedValidationRouteAccess::dispatch(
			$request->route()->uri,
			self::class,
			$request->server('REMOTE_ADDR'),
			($request->user()) ? $request->user()->device_hash : ''
		);
		return $this->forbidden();
    }
}
