<?php

namespace App\Http\Middleware;

use App\Http\Middleware\MiddlewareBase;
use App\Jobs\Channels\PublicChannels\IsUserBanned;
use App\Jobs\Logs\LogFailedValidationRouteAccess;
use App\Traits\Strings\Encryption\UnlockEncryptedRequestKeysTraits;
use App\Transports\Information\ID;

use Closure;

class IsUserBannedFromChannelKnocks extends MiddlewareBase
{
	use UnlockEncryptedRequestKeysTraits;
	
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
		$keys = $this->unlockRequestHeaderKeys($request);
		if (! IsUserBanned::dispatchNow(
			new ID($request->input('channelId'), $request->input('channelClusterId')),
			$keys->userLookupHash
		))
		{
			return $next($request);
		}
		LogFailedValidationRouteAccess::dispatch(
			$request->route()->uri,
			self::class,
			$request->server('REMOTE_ADDR'),
			($request->user()) ? $request->user()->device_hash : ''
		);
		return $this->forbidden();
    }
}
