<?php

namespace App\Interfaces\Repositories;

use App\Enums\Devices\DeviceStatusesEnum;
use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Traits\Authentication\AuthenticationHelperTraits;
use App\Transports\Information\ID;

class AuthenticationAbstractRepository extends BaseReplicatedAbstractRepository
{
	use AuthenticationHelperTraits;
	
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'view_bearer_user_auth';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param type $email
	 * @param type $deviceHash
	 * @return object
	 */
	public function getUser($email, $deviceHash)
	{
		return $this->filterByFields([
			'email_hash' => $this->generateEmailHash($email),
			'device_hash' => $deviceHash
		])->first();
	}
	
	/**
	 * 
	 * @param ID $userId
	 * @param string $excludeDeviceHash
	 * @return recordset
	 */
	public function getUserDevices(ID $userId, $excludeDeviceHash = '')
	{
		$query = $this->filterByFields([
			'id'			=> $userId->id,
			'cluster_id'	=> $userId->clusterId
		])
		->select('device_hash', 'device_description', 'device_user_agent', 'device_status')
		->where('device_status', '<>', DeviceStatusesEnum::DENIED);
		if ($excludeDeviceHash)
		{
			$query->where('device_hash', '<>', $excludeDeviceHash);
		}
		$records = $query->get();
		return $records;
	}
}