<?php

namespace App\Interfaces\Repositories\Base;

use Carbon\Carbon;
use DB;
use PDO;
use Str;

class BaseAbstractRepositoryUtilities
{
	//################################################################################
	//################################################################################
	//################################################################################
	// 	CONSTANTS

	/**
	 * @const DATABASE_TIMESTAMP_FORMAT
	 */
	const DATABASE_TIMESTAMP_FORMAT = 'Y-m-d H:i:s';
	const CARBON_UNIX_TIMESTAMP_FORMAT = 'X';
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * @param integer $number
	 * @return integer
	 */
	protected function getMaxNumberRecords($number = FALSE)
	{
		return ($number) ? $number : config('performance.maxNumberRecords');
	}
	
	/**
	 * @return string
	 */
	protected function getServerTimeField()
	{
		return DB::raw('now() as server_time');
	}

	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param type $days
	 * @param type $date
	 * @return string
	 */
	public function addDays($days, $date = '')
	{
		$workingDate = ($date) ? (new Carbon($date)) : (new Carbon());
		return $this->formatDate($workingDate->addDays($days)->isoFormat(self::CARBON_UNIX_TIMESTAMP_FORMAT));
	}
	
	/**
	 * 
	 * @param string $expiresAt
	 * @return string|null
	 */
	public function expirationDate($expiresAt = '')
	{
		return ($expiresAt)
			? $this->fromJSTimestamp($expiresAt)
			: null;
	}

	/**
	 * @method formatDate
	 * @param string $date
	 * @return string
	 */
	public function formatDate($date = '')
	{
		return ($date)
			? date(self::DATABASE_TIMESTAMP_FORMAT, $date)
			: date(self::DATABASE_TIMESTAMP_FORMAT);
	}
	
	/**
	 * 
	 * @param integer $timestamp
	 * @return string
	 */
	public function fromJSTimestamp($timestamp)
	{
		return $this->formatDate((int)($timestamp / 1000.0));
	}
	
	
	public function fromUrlFormattedTimestamp($timestamp)
	{
		return str_replace('_', ' ', str_replace('.', ':', $timestamp));
	}
	
	/**
	 * 
	 * @param type $configKey
	 * @return string
	 */
	public function generateToken($configKey = 'default')
	{
		$minLength = (int)config('tokens.' . $configKey . '.length.min');
		$maxLength = (int)config('tokens.' . $configKey . '.length.max');
		return Str::random((int)rand($minLength, $maxLength));
	}

	/**
	 * @method getCurrentTimestamp
	 * @return string
	 */
	public function getCurrentTimestamp()
	{
		return $this->formatDate();
	}
	
	/**
	 * 
	 * @param string $configKey
	 * @return string
	 */
	public function tokenExpirationDate($configKey = 'default')
	{
		$dateString = '+' . config('tokens.' . $configKey . '.expirationHours') . ' hours';
		return $this->formatDate(strtotime($dateString));
	}
}