<?php

namespace App\Interfaces\Repositories;

use App\Transports\Information\ID;
use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;

use Carbon;

class ChannelAbstractRepository extends BaseReplicatedAbstractRepository
{
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'channels';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	public function getPublicChannelInfo(ID $id)
	{
		return $this->filterById($id)
			->select('searchable_title', 'searchable_description', 'public_key')
			->where(['searcbable' => true])
			->first();
	}
	
	
	/**
	 * 
	 * @param integer $numberDays
	 * @param integer $maxRecords
	 * @return recordset
	 */
	public function getRecentPublicChannels($numberDays = 5, $maxRecords = 0)
	{
		return $this->filterByActive()
			->where('created_at', '>=', $this->formatDate(Carbon::now()->subDays($numberDays)))
			->where(['searcbable' => true])
			->limit($this->getMaxNumberRecords($maxRecords))
			->orderBy('created_at', 'desc')
			->get();
	}
	
	/**
	 * 
	 * @param integer $maxRecords
	 * @return recordset
	 */
	public function getPopularPublicChannels($maxRecords = 0)
	{
		
	}
	
	/**
	 * 
	 * @param string $search
	 * @param integer $maxRecords
	 * @return recordset
	 */
	public function searchPublicChannels($search, $maxRecords = 0)
	{
		return $this->filterByActive()
			->select('id', 'cluster_id', 'searchable_title', 'searchable_description')
			->where(['searchable' => true])
			->where(function ($query) use ($search) {
				$query->whereRaw('instr(searchable_title, ?) > 0', [$search]);
				$query->orWhereRaw('instr(searchable_description, ?) > 0', [$search]);
			})
			->limit($this->getMaxNumberRecords($maxRecords))
			->orderBy('created_at', 'desc')
			->get();
	}
}