<?php

namespace App\Interfaces\Repositories;

use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Transports\Information\ID;

class ChannelKnockBanAbstractRepository extends BaseReplicatedAbstractRepository
{
    /**
     * @var boolean replicatedTable
     */
    protected $replicatedTable = true;
    
    /**
     * @var string $table
     */
    protected $table = 'channel_knock_bans';
	
	/**
	 * 
	 * @param ID $channelId
	 * @param integer $page
	 * @param integer $maxNumber
	 * @return recordset
	 */
	public function getChannelBans($channelId, $page = 0, $maxNumber = 0)
	{
		$numberRecordsPerPage = $this->getMaxNumberRecords($maxNumber);
		return $this->filterByFields([
			'channel_cluster_id'	=> $channelId->clusterId,
			'channel_id'			=> $channelId->id
		])->skip($page * $numberRecordsPerPage)
		->limit($numberRecordsPerPage)
		->get();
	}
	
	/**
	 * 
	 * @param ID $channelId
	 * @param string $userHash
	 * @return integer
	 */
	public function isUserBanned($channelId, $userHash)
	{
		return $this->filterByFields([
			'channel_cluster_id'	=> $channelId->clusterId,
			'channel_id'			=> $channelId->id,
			'user_hash'				=> $userHash
		])->where('expires_at', '>', $this->formatDate())
		->count();
	}
	
	/**
	 * 
	 * @param ID $channelId
	 * @param ID $knockId
	 * @return integer
	 */
	public function removeBan(ID $channelId, ID $knockId)
	{
		$timestamp = $this->getCurrentTimestamp();
		return $this->filterByFields([
			'channel_cluster_id'	=> $channelId->clusterId,
			'channel_id'			=> $channelId->id,
			'cluster_id'			=> $knockId->clusterId,
			'id'					=> $knockId->id
		])->update([
			'active'				=> false,
			'deleted_at'			=> $timestamp,
			'updated_at'			=> $timestamp
		]);
	}
}