<?php

namespace App\Interfaces\Repositories;

use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Transports\Information\ID;

class ChannelKnocksAbstractRepository extends BaseReplicatedAbstractRepository
{
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'channel_knocks';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * @method filterByActive
	 * @return QueryBuilder
	 */
	protected function filterByActive()
	{
		return parent::filterByActive()
			->where('expires_at', '>', $this->formatDate());
	}
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * @method delete
	 * @param ID $id
	 * @return @TODO
	 */
	public function delete(ID $id)
	{
		return $this->update($id, [
			'active'		=> false,
			'deleted_at'	=> $this->getCurrentTimestamp()
		]);
	}
	
	/**
	 * 
	 * @param string $token
	 * @param ID $channelId
	 * @return integer
	 */
	public function deleteByToken($token, ID $channelId)
	{
		$timestamp = $this->getCurrentTimestamp();
		return $this->filterByFields([
				'token'					=> $token,
				'channel_cluster_id'	=> $channelId->clusterId,
				'channel_id'			=> $channelId->id
			])
			->update([
				'active'		=> false,
				'deleted_at'	=> $timestamp,
				'updated_at'	=> $timestamp
			]);
	}

	/**
	 *
	 * @param string $token
	 * @param ID $channelId
	 * @return mixed
	 */
	public function getByToken($token, ID $channelId)
	{
		return $this->filterByFields([
			'token'					=> $token,
			'channel_cluster_id'	=> $channelId->clusterId,
			'channel_id'			=> $channelId->id
		])->first();
	}
	
	/**
	 * 
	 * @param ID $channelHash
	 * @param integer $maxNumber
	 * @param integer $page
	 * @return recordset
	 */
	public function getChannelKnocksForReview(ID $channelId, $maxNumber = 0, $page = 0)
	{
		$number = $this->getMaxNumberRecords($maxNumber);
		return $this->filterByActive()
			->where('channel_id', '=', $channelId->id)
			->where('channel_cluster_id', '=', $channelId->clusterId)
			->skip($page * $number)
			->take($number)
			->get();
	}
	
	/**
	 * 
	 * @param string $emailHash
	 * @return array
	 */
	public function getMyChannelKnocks($emailHash)
	{
		
		
	}
	
	/**
	 * @method insert
	 * @param array $fields
	 * @return mixed
	 */
	public function insert(Array $fields = [])
	{
		$fields['expires_at'] = $this->tokenExpirationDate('knocks');
		return parent::insert($fields);
	}
}