<?php

namespace App\Interfaces\Repositories;

use App\Enums\Channels\ChannelRolesEnum;
use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;

use DB;

class ChannelMemberAbstractRepository extends BaseReplicatedAbstractRepository
{
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'channel_members';
	
	/**
	 * 
	 * @param string $channelHash
	 * @param string $userHash
	 * @param string $role
	 */
	public function changeMemberRole($channelHash, $userHash, $role = ChannelRolesEnum::member)
	{
		return DB::table($this->table)
			->where('active', true)
			->where('channel_hash', $channelHash)
			->where('user_hash', $userHash)
			->update(['roles' => $role]);
	}
	
	/**
	 * 
	 * @param string $userHash
	 * @param string $channelHash
	 * @param string $fieldName
	 * @return string
	 */
	public function getMemberField($userHash, $channelHash, $fieldName = 'encrypted_channel_info')
	{
		return $this->filterByFields(['user_hash' => $userHash, 'channel_hash' => $channelHash])
			->select($fieldName)
			->first()
			->$fieldName;
	}
	
	
	public function getMembers($channelHash)
	{
		return $this->filterByFields(['channel_hash' => $channelHash])
			->select('user_hash', 'encrypted_user_info', 'roles')
			->get();
	}
	
	/**
	 * 
	 * @param string $userHash
	 * @return recordset
	 */
	public function getMemberships($userHash)
	{
		return $this->filterByFields(['user_hash' => $userHash])
			->get();
	}
	
	/**
	 * 
	 * @param string $channelHash
	 * @param string $userLookupHash
	 * @return boolean
	 */
	public function isAdmin($channelHash, $userLookupHash)
	{
		return $this->filterByFields([
			'user_hash'		=> $userLookupHash,
			'channel_hash'	=> $channelHash
			])
			->whereRaw('instr(roles, ?) > 0', [ChannelRolesEnum::administrator])
			->get()->count();
	}
	
	/**
	 * 
	 * @param string $channelHash
	 * @param string $userLookupHash
	 * @return boolean
	 */
	public function isMember($userLookupHash, $channelHash)
	{
		return $this->filterByFields([
			'user_hash'		=> $userLookupHash,
			'channel_hash'	=> $channelHash,
		])->get()->count();
	}
	
	/**
	 * 
	 * @param string $channelHash
	 * @param string $userHash
	 * @return boolean
	 */
	public function removeMember($channelHash, $userHash)
	{
		return $this->filterByFields([
			'channel_hash'	=> $channelHash,
			'user_hash'		=> $userHash
		])->update(['deleted_at' => $this->getCurrentTimestamp()]);
	}
	
	/**
	 * 
	 * @param string $channelHash
	 * @param string $userHash
	 * @param string $roles
	 */
	public function updateRoles($channelHash, $userHash, $roles)
	{
		return $this->filterByFields([
			'channel_hash'	=> $channelHash,
			'user_hash'		=> $userHash
		])->update(['roles' => $roles]);
	}
}