<?php

namespace App\Interfaces\Repositories;

use App\Facades\Utilities\Users\Preferences;
use App\Interfaces\Constants\PreferencesConstants;
use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Transports\Information\ID;

class ContentAbstractRepository extends BaseReplicatedAbstractRepository
 implements PreferencesConstants
{
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'content';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * @method initializeTable
	 * @return QueryBuilder
	 */
	protected function initializeTable()
	{
		return parent::initializeTable()
			->orderBy('created_at', 'desc');
	}
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param string $channelHash
	 * @param string $timestamp
	 * @return Collection
	 */
	public function getMessages($channelHash, $timestamp = '')
	{
		$pageSize = Preferences::getContentPageSize();
		$query = $this->filterByActive()
			->select('id', 'cluster_id', 'author_hash', 'created_at')
			->where('channel_hash', $channelHash);
		if ($timestamp)
		{
			$query->where('created_at', '<', $this->fromUrlFormattedTimestamp($timestamp));
		}
		$content = $query->take($pageSize)
			->orderBy('created_at', 'desc')
			->get();
		return $content;
	}
	
	/**
	 * 
	 * @param ID $id
	 * @return recordset
	 */
	public function getEncryptedDataById(ID $id)
	{
		return $this->filterById($id)
			->select('encrypted_content', 'encrypted_meta')
			->first();
	}
	
	/**
	 * @method insert
	 * @param array $fields
	 * @return mixed
	 */
	public function insert(Array $fields = [])
	{
		$fields['created_at'] = $this->getCurrentTimestamp();
		return parent::insert($fields);
	}
	
	/**
	 * 
	 * @param string $userLookupHash
	 * @param ID $contentId
	 * @return boolean
	 */
	public function userIsAuthor($userLookupHash, ID $contentId)
	{
		$content = $this->getById($contentId);
		return $content->author_hash == $userLookupHash;
	}
}