<?php

namespace App\Interfaces\Repositories;

use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Interfaces\Constants\PreferencesConstants;

class DeviceConfirmationAbstractRepository extends BaseReplicatedAbstractRepository
 implements PreferencesConstants
{
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'device_confirmations';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * @method initializeTable
	 * @return QueryBuilder
	 */
	protected function initializeTable()
	{
		return parent::initializeTable()
			->where('used', '=', 'false');
	}
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param string $deviceHash
	 * @param string $token
	 * @return boolean
	 */
	public function useToken($deviceHash, $token)
	{
		$this->filterByFields([
			'device_hash'	=> $deviceHash,
			'token'			=> $token
		])->update(['used' => true]);
	}
	
	/**
	 * 
	 * @param string $emailHash
	 * @return string
	 */
	public function createConfirmationRecord($deviceHash)
	{
		$token = $this->generateToken();
		$this->insert([
			'token' => $token,
			'device_hash' => $deviceHash
		]);
		return $token;
	}
	
	/**
	 * @method insert
	 * @param array $fields
	 * @return mixed
	 */
	public function insert(Array $fields = [])
	{
		$fields['expires_at'] = $this->tokenExpirationDate();
		return parent::insert($fields);
	}
}