<?php

namespace App\Interfaces\Repositories;

use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;

class InvitationAbstractRepository extends BaseReplicatedAbstractRepository
{
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'invitations';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * @method filterByActive
	 * @return QueryBuilder
	 */
	protected function filterByActive()
	{
		return parent::filterByActive()
			->where('expires_at', '>', $this->formatDate());
	}
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS

	/**
	 *
	 * @param string $token
	 * @param string $email_hash
	 * @return mixed
	 */
	public function getInvitation($token, $email_hash)
	{
		$invitation = $this->filterByFields([
			'token'			=> $token,
			'email_hash'	=> $email_hash
		])->get();
		if ($invitation && count($invitation))
		{
			return $invitation[0];
		}
	}
	
	/**
	 * 
	 * @param string $emailHash
	 * @return array
	 */
	public function getMyInvitations($emailHash)
	{
		return $this->filterByFields(['email_hash' => $emailHash])
			->get();
	}
	
	/**
	 * @method insert
	 * @param array $fields
	 * @return mixed
	 */
	public function insert(Array $fields = [])
	{
		$fields['expires_at'] = $this->tokenExpirationDate();
		return parent::insert($fields);
	}
}