<?php

namespace App\Interfaces\Repositories;

use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;
use App\Transports\Information\ID;

class PreferenceAbstractRepository extends BaseReplicatedAbstractRepository
{
    // ##########
    // standard table info
    /**
     * @var string $table
     */
    protected $table = 'preferences';
	
	
	/**
	 * 
	 * @param string $emailHash
	 * @param string $deviceHash
	 * @return recordset
	 */
	public function getPreferences($emailHash, $deviceHash)
	{
		return $this->initializeTable()
			->where('email_hash', '=', $emailHash)
			->where(function ($query) use ($deviceHash){
				$query->where('device_hash', '=', $deviceHash)
					->orWhere('device_hash', 'is', 'null');
			})->get();
	}
	
	/**
	 * 
	 * @param string $emailHash
	 * @param string $ddeviceHash
	 * @param string $preference
	 * @param mixed $value
	 * @return recordset
	 */
	public function savePreference($emailHash, $deviceHash, $preference, $value)
	{
		if ($preferenceRecord = $this->filterByFields([
				'email_hash' =>  $emailHash,
				'device_hash' => $deviceHash,
				'preference' => $preference
			])->first()
		)
		{
			return $this->update(
				new ID($preferenceRecord->id, $preferenceRecord->cluster_id),
				['value' => $value]
			);
		}
		else
		{
			return $this->insert([
				'email_hash' =>  $emailHash,
				'device_hash' => $deviceHash,
				'preference' => $preference,
				'value' => $value
			]);
		}
	}
}