<?php

namespace App\Interfaces\Repositories;

use App\Traits\Authentication\AuthenticationHelperTraits;

use App\Interfaces\Repositories\Base\BaseReplicatedAbstractRepository;

class UserAbstractRepository extends BaseReplicatedAbstractRepository
{
	use AuthenticationHelperTraits;
	
    /**
     * @var string $table
     */
	protected $table = 'users';
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PROTECTED METHODS
	
	/**
	 * @method filterByActive
	 * @return QueryBuilder
	 */
	protected function filterByActive()
	{
		return parent::filterByActive()
			->where(function ($query){
				$query->whereNull('expires_at');
				$query->orWhere('expires_at', '>', $this->formatDate());
			});
	}
	
	
	//################################################################################
	//################################################################################
	//################################################################################
	// PUBLIC METHODS
	
	/**
	 * 
	 * @param string $emailHash
	 * @return void
	 */
	public function emailConfirmed($emailHash)
	{
		$this->filterByFields(['email_hash' => $emailHash])
			->update(['email_verified_at' => $this->formatDate()]);
	}

	/**
	 * @method getByEmail
	 * @param string $email
	 */
	public function getByEmail($email)
	{
		return $this->getByEmailHash($this->generateEmailHash($email));
	}
	
	/**
	 * 
	 * @param string $hash
	 */
	public function getByEmailHash($hash)
	{
		return $this->filterByFields(['email_hash' => $hash])->first();
	}
	
	/**
	 * 
	 * @param type $emailHash
	 * @return type string
	 */
	public function getEncryptedInfo($emailHash)
	{
		return $this->filterByFields(['email_hash' => $emailHash])
			->select('encrypted_info')
			->first()->encrypted_info;
	}
	
	/**
	 * 
	 * @param array $emailHashes
	 * @return array of records
	 */
	public function getPublicKeys(array $emailHashes)
	{
		return $this->initializeTable()
			->select('email_hash', 'public_key')
			->whereIn('email_hash', $emailHashes)
			->get();
	}
	
	/**
	 * @method insertStub
	 * @param array $fields
	 * @return mixed
	 */
	public function insertStub(Array $fields = [])
	{
		$fields['expires_at'] = $this->tokenExpirationDate();
		return parent::insert($fields);
	}
}