<?php

namespace App\Interfaces\Utilities\String;

//use Illuminate\Support\Facades\Redis;
use Illuminate\Redis\RedisManager as LaravelRedisManager;

abstract class GuidAbstractUtility
{
	/**
	 * @var $redis;
	 */
	protected $redis;

	/**
	 * @method getCluster
	 * @return integer
	 */
	protected function getCluster()
	{
		return intval(env('SYNCHRONIZATION_CLUSTER_ID'));
	}

	/**
	 * @method getClusterIncrementer
	 * @return integer
	 */
	protected function getClusterIncrementer()
	{
		intval($this->redis->get('system.synchronization.clusters.guid'));
	}

	/**
	 * @method construct
	 * @param LaravelRedisManager $redis
	 */
	public function __construct(LaravelRedisManager $redis)
	{
		$this->redis = $redis;
	}

	/**
	 * @method getSerial
	 * @return integer
	 */
	public function getSerial()
	{
		return time() - intval(env('GUID_GENERATION_START_TIMESTAMP'));
	}

	/**
	 * @method updateClusterIncrementer
	 * @param integer $seedValue
	 * @return integer
	 */
	public function updateClusterIncrementer($seedValue)
	{
		$this->redis->set(
			'system.synchronization.clusters.guid',
			($seedValue + 1) % 65536
		);
	}

	/**
	 * @method make
	 * @return integer
	 */
	public function make()
	{
		$clusterCounter = $this->getClusterIncrementer();
		$this->updateClusterIncrementer($clusterCounter);
		$clusterId = $this->getCluster();
		$prefix = $this->getSerial();
		return
			($prefix << 32)
			+ ($clusterId << 16)
			+ $clusterCounter;
	}
}