<?php

namespace App\Traits\Authentication;

use App\Traits\Strings\Encryption\EncryptionHelperTraits;
use App\Traits\Strings\Hashes\HashGeneratorHelperTraits;

use Hash;

/**
 * Description of AuthenticationHelperTraits
 *
 * @author Scott Kerns
 */
trait AuthenticationHelperTraits {
	use EncryptionHelperTraits, HashGeneratorHelperTraits;
	
	/**
	 * 
	 * @param mixed $emails
	 * @return mixed
	 */
	public function convertEmailsToHash($emails)
	{
		return is_array($emails)
			? array_map(
				function ($email)
				{
					return $this->generateEmailHash($email);
				},
				$emails
			)
			: $this->generateEmailHash($emails);
	}
	
	/**
	 * 
	 * @param string $email
	 * @return string
	 */
	public function generateEmailHash($email)
	{
		return $this->hash($email, '', [env('HASHING_APPENDING_SEED_EMAIL')]);
	}
	
	/**
	 * 
	 * @param string $password
	 * @return string
	 */
	public function generatePasswordHash($password)
	{
		return Hash::make($password);
	}
	
	/**
	 * 
	 * @param string $userRecord
	 * @param string $providedPassword
	 * @param string $providedPhrase
	 * @return boolean
	 */
	public function testAuthenticationHashes($userRecord, $providedPassword, $providedPhrase)
	{
		return $this->testPasswordHash($providedPassword, $userRecord->password)
			&& $this->testPasswordHash($providedPhrase, $userRecord->phrase_hash);
	}
	
	/**
	 * 
	 * @param string $password
	 * @param string $passwordHash
	 * @return string
	 */
	public function testPasswordHash($password, $passwordHash)
	{
		return Hash::check($password, $passwordHash);
	}
	
	/**
	 * 
	 * @param string $userRecord
	 * @param string $loginUnlockString
	 * @return object
	 */
	public function unlockSigninAuthKeys($userRecord, $loginUnlockString)
	{
		$userEncryptionKey = $this->decryptByStringKey(
			$userRecord->user_encryption_key,
			$loginUnlockString
		);
		$apiToken = 'Bearer ' . $this->decryptByStringKey(
			$userRecord->encrypted_api_token,
			$userEncryptionKey
		);
		return (object)compact('userEncryptionKey', 'apiToken');
	}
}
