<?php

namespace App\Traits\Objects;

use App\Traits\Strings\Encryption\EncryptionGeneratorHelperTraits;
use App\Traits\Strings\Encryption\EncryptionHelperTraits;
use App\Traits\Strings\Hashes\HashGeneratorHelperTraits;
use App\Transports\Information\EncryptionKeys;
use App\Transports\Information\Hashes;
use App\Transports\Security\NewRecordStrings;

/**
 * Description of CreationHelperTraits
 *
 * @author Scott Kerns
 */
trait CreationHelperTraits {
	use EncryptionGeneratorHelperTraits, EncryptionHelperTraits, HashGeneratorHelperTraits;
	
	/**
	 * 
	 * @param string $type
	 * @return Hashes
	 */
	public function generateHashStrings($type)
	{
		$hashSeed = $this->generateHashSeed();
		return new Hashes(
			$hashSeed,
			$this->generateLookupHash($type, $hashSeed, [env('HASHING_APPENDING_SEED_RECORD_LOOKUPS')])
		);
	}
	
	/**
	 * 
	 * @param string $type
	 * @param boolean $includeStringKey
	 * @return NewRecordStrings
	 */
	public function generateSecurityStrings($type)
	{
		return new NewRecordStrings(
			$this->generateHashStrings($type),
			new EncryptionKeys(
				$this->generateKeyPair(),
				$this->generateStringKey(),
				$this->generateStringKey()
			)
		);
	}
}
