<?php

namespace App\Traits\Strings\Encryption;

use App\Traits\Strings\Encryption\EncryptionHelperTraits;
use App\Transports\Encryption\Users\UserDataKeys;
use App\Transports\Information\EncryptionKeys;
use App\Transports\Encryption\Unlock\UnlockCredentials;

/**
 * Description of UnlockCredentialsHelperTraits
 *
 * @author Scott Kerns
 */
trait UnlockCredentialsHelperTraits {
	use EncryptionHelperTraits;
	
	/**
	 * 
	 * @param array $userRecord
	 * @param UserDataKeys $userDataKeys
	 * @return object
	 */
	protected function unlockUserRecordInfo(Array $userRecord, UserDataKeys $userDataKeys)
	{
		return $this->decryptByStringKey(
			$userRecord['encrypted_info'],
			$userDataKeys->userEncryptionKey,
			true
		);
	}
	
	/**
	 * 
	 * @param object $userInfo
	 * @return UnlockCredentials
	 */
	public function getUnlockCredentialsFromUserInfo($userInfo)
	{
		return new UnlockCredentials(
			$userDataKeys->userLookupHash,
			$decryptedUserInfo->keys->privateKey
		);
	}
	
	/**
	 * 
	 * @param object $userInfo
	 * @param string $encryptedInfo
	 * @return object
	 */
	public function unlockEncryptedInfoWithUserPrivateKey($userInfo, $encryptedInfo)
	{
		return $this->unsealWithPrivateKey(
			$encryptedInfo,
			$userInfo->keys->keyPair->privateKey,
			true
		);
	}
	
	/**
	 * 
	 * @param array $userRecord
	 * @param UserDataKeys $userDataKeys
	 * @param string $encryptedInfo
	 * @return object
	 */
	public function unlockUserInfoThenEncryptedInfo(Array $userRecord, UserDataKeys $userDataKeys, $encryptedInfo)
	{
		$userInfo = $this->unlockUserRecordInfo($userRecord, $userDataKeys);
		return $this->unlockEncryptedInfoWithUserPrivateKey($userInfo, $encryptedInfo);
	}
}
