<?php

namespace App\Traits\Strings\Hashes;

use App\Traits\Strings\Utilities\StringUtilityTraits;

/**
 * Description of GeneratorHelperTraits
 *
 * @author Scott Kerns
 */
trait HashGeneratorHelperTraits {
	use StringUtilityTraits;
	
	/**
	 * 
	 * @param mixed $target
	 * @param mixed $seed
	 * @return string
	 */
	protected function generatorHelperMakeHash($target, $seed = '')
	{
		return hash(
			config('hashing.genericHashDriver'),
			$this->arrayToString($target)
			. $this->arrayToString($seed)
		);
	}
	
	/**
	 * 
	 * @param mixed $keys
	 * @param mixed $seed
	 * @param mixed $appendingSeeds
	 * @return string
	 */
	public function hash($keys, $seed = '', $appendingSeeds = '')
	{
//		$baseString = $this->arrayToString($keys);
//		$baseHash = $this->arrayToString($seed);
//		$hashes = [$this->generatorHelperMakeHash($baseString, $baseHash)];
//		foreach ($this->stringToArray($appendingSeeds) as $additionalSeed)
//		{
//			$hashes[] = $this->generatorHelperMakeHash(
//				$baseString,
//				$baseHash . $this->arrayToString($additionalSeed)
//			);
//		}
//		return implode('', $hashes);
		return $this->generatorHelperMakeHash(
			$this->arrayToString($keys),
			$seed . $this->arrayToString($appendingSeeds)
		);
	}
	
	/**
	 * 
	 * @param string $type
	 * @param string $seed
	 * @param array $additionalHashSeeds
	 * @param integer $timestamp
	 * @param integer $random
	 * @param integer $clusterId
	 * @return string
	 */
	public function generateLookupHash(
		$type,
		$seed,
		Array $additionalHashSeeds = [],
		$timestamp = 0,
		$random = 0,
		$clusterId = 0
	)
	{
		return $this->hash(
			[
				$type,
				$this->formatIntegerForHash($timestamp ? $timestamp : time()),
				$this->formatIntegerForHash($random ? $random : rand(0, PHP_INT_MAX)),
				$this->formatIntegerForHash($clusterId ? $clusterId : env('SYNCHRONIZATION_CLUSTER_ID'))
			],
			$seed,
			$additionalHashSeeds
		);
	}
	
	/**
	 * @return string
	 */
	public function generateHashSeed()
	{
		return $this->hash($this->generateRandomString(
			config('hashing.hashSeedMinLength'),
			config('hashing.hashSeedMaxLength')
		));
	}
}
