<?php

namespace App\Traits\Strings\Utilities;

use Str;

/**
 * Description of UtilityTraits
 *
 * @author Scott Kerns
 */
trait StringUtilityTraits {
	/**
	 * 
	 * @param mixed $target
	 * @param string $separator
	 * @return string
	 */
	public function arrayToString($target, $separator = '')
	{
		return (is_array($target))
			? implode($separator, $target)
			: $target;
	}
	
	/**
	 * 
	 * @param string $serializedString
	 * @return mixed
	 */
	public function deSerialize($serializedString)
	{
		return json_decode($serializedString);
	}
	
	/**
	 * 
	 * @param integer $integer
	 * @return string
	 */
	public function formatIntegerForHash($integer)
	{
		return str_pad(
			$integer,
			ceil(log10(PHP_INT_MAX)),
			'0',
			STR_PAD_LEFT
		);
	}
	
	/**
	 * 
	 * @param integer $minLength
	 * @param integer $maxLength
	 * @param boolean $toBase64
	 * @return string
	 */
	public function generateRandomString($minLength = 256, $maxLength = 2048, $toBase64 = false)
	{
		$bytes = openssl_random_pseudo_bytes((int)rand($minLength, $maxLength));
		return ($toBase64) ? base64_encode($bytes) : $bytes;
	}
	
	/**
	 * 
	 * @param mixed $targetToSerialize
	 * @return string
	 */
	public function serialize($targetToSerialize)
	{
		return json_encode(
			$targetToSerialize,
			JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_NUMERIC_CHECK
		);
	}
	
	/**
	 * 
	 * @param mixed $target
	 * @return array
	 */
	public function stringToArray($target)
	{
		return ($target)
			? (
				is_array($target)
				? $target
				: [$target]
			)
			: [];
	}
}
