const textEditorDefinitions = {
	preProcessors: [
		{token: '---', handler: 'HoriziontalLine', description: 'Line'}
	],

	// token, aliases, insertionHandler (character, inline, block), options: [], renderHandler (character, inline, block), descriptionLookup
	tokenConversions: {
		beginsBlock: function (code)
		{
			return code + '_BEGIN';
		},
		codeToCommand: function (code)
		{
			return code.substr(1, code.length - 2);
		},
		commandToCode: function (command)
		{
			return '=' + command + '(';
		},
		endsBlock: function (code)
		{
			return code + '_END';
		}
	},

	topLevelCommands: [
		{
			token: 'Bold',
			aliases: ['B'],
			htmlElement: 'b',
			insertionHandler: 'inline',
			usage: '=Bold("text")'
		},
		{
			token: 'Character',
			aliases: ['Char', 'C'],
			htmlElement: 'span',
			insertionHandler: 'character',
			usage: ''
		},
		{
			token: 'Comments',
			aliases: ['Comment'],
			htmlElement: '',
			insertionHandler: 'character',
			usage: ''
		},
		{
			token: 'Image',
			aliases: ['Img', 'I'],
			htmlElement: 'img',
			insertionHandler: 'character',
			jsonNode: {
				attributeMap: {src: 'link', alt: 'alt'},
				name: 'image'
			},
			renderHandler: 'character',
			usage: '=Image(link="https://www.nasa.gov/images/content/683916main_DavidCartierSr-RiverMirror-orig_full.jpg" alt="NASA - VISIONS: Seeing the Aurora in a New Light")'
		},
		{
			token: 'Italic',
			aliases: ['It'],
			htmlElement: 'i',
			insertionHandler: 'inline',
			renderHandler: 'inline',
			usage: '=Italics("some italic text")'
		},
		{
			token: 'Link',
			aliases: ['L'],
			htmlElement: 'a',
			insertionHandler: 'character',
			jsonNode: {
				attributeMap: {link: 'link', text: 'text'},
				name: 'link'
			},
			renderHandler: 'character',
			usage: '=Link(link="https://www.nasa.gov/" text="NASA")'
		},
		{
			token: 'NewLine',
			aliases: [],
			htmlElement: 'br',
			insertionHandler: 'character',
			renderHandler: 'character',
			usage: ''
		},
		{
			token: 'Underline',
			aliases: ['UL', 'U'],
			htmlElement: 'u',
			insertionHandler: 'inline',
			renderHandler: 'inline',
			usage: '=Underline("underlined text")'
		}
	]
}

textEditorDefinitions.availableCommands = (function  () {
	const commands = [];
	textEditorDefinitions.topLevelCommands.forEach((command) => {
		commands.push({token: command.token, handler: command.token});
		command.aliases.forEach((alias) => {
			commands.push({token: alias, handler: command.token});
		});
	});
	return commands;
}());

export default textEditorDefinitions