import definitions from './definitions.js';

const availableCommands = definitions.availableCommands;

/**
 * @param {object} options
 * @param {string} name
 */
const optionsFallback = function (options, name) {
	return options[name] || options.default;
},

/**
 * 
 * @param {object} domContainer 
 * @param {string} elementName 
 * @param {object|string} options 
 */
optionsToAttributes = function (domContainer, elementName, options) {
	const element = document.createElement(elementName),
		attributeName = function (name)
		{
			return (name == 'default' || name == 'text') ? 'innerText' : name;
		}
	if (typeof options === 'object')
	{
		for (let name in options)
		{
			element[attributeName(name)] = options[name];
		}
	}
	else
	{
		element.innerText = options;
	}
	domContainer.appendChild(element);
},

handlers = {
	Bold: function (domContainer, node) {
		optionsToAttributes(domContainer, 'b', node.options);
	},
	Character: function (domContainer, node) {
		optionsToAttributes(domContainer, 'span', {innerHTML: optionsFallback(node.options, 'code')});
	},
	Comments: function (domChanged, node) { },
	// Code: {type: 'block'},
	content: function (domContainer, node) {
		optionsToAttributes(domContainer, 'span', {innerText: node.data});
	},
	Emoticon: function (domContainer, character) {

	},
	HoriziontalLine: function (domContainer) {
		optionsToAttributes(domContainer, 'hr', {});
	},
	Image: function (domContainer, node) {
		if (nodeValidators(['link'], node)) {
			optionsToAttributes(domContainer, 'img', {src: node.options.link, alt: node.options.alt});
		}
	},
	Italic: function (domContainer, node) {
		optionsToAttributes(domContainer, 'i', node.options);
	},
	Link: function (domContainer, node) {
		if (nodeValidators(['link', 'text'], node)) {
			optionsToAttributes(domContainer, 'a', {href: node.options.link, innerText: node.options.text, target: '_blank'});
		}
	},
	// List: {type: 'block'},
	NewLine: function (domContainer) {
		optionsToAttributes(domContainer, 'br', {});
	},
	// Table: {type: 'block'}
	Underline: function (domContainer, node) {
		optionsToAttributes(domContainer, 'u', {innerText: optionsFallback(node.options, 'text')});
	}
},

/**
 * @param {Node} node
 * @return string
 */
getHandler = function (node) {
	let handler = _.find(availableCommands, (command) => {
		return node.handlerName == command.token;
	});
	return (handler) ? handler.handler : 'content';
},

/**
 * @param {array | string} requiredAttributes
 * @param {Node} node
 * @return boolean
 */
nodeValidators = function (requiredAttributes, node) {
	let valid = true;
	requiredAttributes.forEach((attribute) => {
		valid = valid && node.options.hasOwnProperty(attribute) && node.options[attribute];
	});
	return valid;
}

const DomBuilder = function (domContainer) {
	this.handle = function (node) {
		handlers[getHandler(node)](domContainer, node);
	}
}

export default DomBuilder