import Node from '../node.js';

export default {
	data()
	{
		return {
			defaultEmojis: [
				{code: '1F31D', description: 'full moon face'},
				{code: '1F3C6', description: 'trophy'},
				{code: '1F440', description: 'eyes'},
				{code: '1F601', description: 'beaming face with smiling eyes'},
				{code: '1F607', description: 'smiling face with halo'},
				{code: '1F609', description: 'winking face'},
				{code: '1F60E', description: 'smiling face with sunglasses'},
				{code: '1F61B', description: 'face with tongue'},
				{code: '1F6A7', description: 'construction'},
				{code: '1F91F', description: 'love-you gesture'}
			],
			emojis: [],
			search: ''
		}
	},

	props: {
		id: {
			required: true,
			type: String
		}
	},

	methods: {
		filteredEmojis(searchTerm)
		{
			if (searchTerm.length > 2)
			{
				const search = searchTerm.toLowerCase();
				return _.filter(this.emojis, (emoji) => {
					return -1 !== emoji.description.indexOf(search);
				});
			}
			return this.defaultEmojis;
		},

		getText(code)
		{
			const s = document.createElement('span');
			s.innerHTML = this.getUnicodeCode(code);
			return s.innerText.charCodeAt(0);
		},

		getUnicodeCode(code)
		{
			return '&#x' + code;
		},

		reset()
		{
			this.search = '';
		},

		selectEmoji(emoji)
		{
			const eventData = {
				command: 'Character',
				options: {code: this.getUnicodeCode(emoji.code)}
			}
			this.$emit('characterSelected', eventData);
		}
	},

	mounted()
	{
		const thisEmojis = this,
			emojiListUrl = this.$utilities.globalConstants.apiEndpoints.editorHelpers.textEditor.emojis();
		thisEmojis.$utilities.ajax.get(emojiListUrl).then((response) => {
			response.data.emojis.forEach((emoji) => {
				thisEmojis.emojis.push(emoji);
			});
		});
		$(this.$el).foundation();
	}
}