import textEditorDefinitions from './definitions.js';

export default {
	data() {
		return {
			content: '',
			editorTabsId: this.$utilities.strings.uniqueStrings.generate('text-editor-tabs'),
			editorTabsWYSIWYGId: this.$utilities.strings.uniqueStrings.generate('text-editor-tabs-wysiwyg'),
			editorTabsSourceId: this.$utilities.strings.uniqueStrings.generate('text-editor-tabs-source'),
			iconIdEmojis: this.$utilities.strings.uniqueStrings.generate('text-editor-emojis'),
			iconIdHelp: this.$utilities.strings.uniqueStrings.generate('text-editor-help'),
			iconIdImages: this.$utilities.strings.uniqueStrings.generate('text-editor-images'),
			iconIdLinks: this.$utilities.strings.uniqueStrings.generate('text-editor-links'),
			textArea: null,
			toolbarIconsTabIndex: -1,
			wysiwygEditor: null
		}
	},

	methods: {
		buildCommand(command, options) {
			return '=' + command + '(' + this.buildOptions(options) + ')';
		},

		buildOptions(options) {
			let optionsArray = [];
			if (options) {
				for (let optionName in options) {
					optionsArray.push(optionName + '="' + options[optionName] + '"');
				}
			}
			return optionsArray.join(' ');
		},

		characterSelected(event) {
			const s = document.createElement('span');
			s.innerHTML = event.options.code;
			const character = s.innerHTML;
			this.insertAsCharacter(this.content, character, this.textArea.selectionStart, this.textArea.selectionEnd);
			this.updatePreview();
		},

		eitherEditorContentChanged()
		{
			this.$emit('input', this.content);
			this.$emit('change', this.content);
		},

		inlineEditorCommand(command) {
			this.updateContent({
				command: command,
				options: {},
				insertionType: 'inline'
			});
		},

		insertAsCharacter(originalContent, newContent, starts, ends) {
			let content = '';
			content += (starts) ? originalContent.substr(0, starts) : '';
			content += newContent;
			content += originalContent.substr(ends);
			this.content = content;
			this.eitherEditorContentChanged();
		},

		insertAsInline(originalContent, command, starts, ends) {
			let content = (starts) ? originalContent.substr(0, starts) : '';
			content += '=' + command + '("' + originalContent.substr(starts, ends - starts) + '")';
			content += originalContent.substr(ends);
			this.content = content;
			this.eitherEditorContentChanged();
		},

		onFocusChange(hasFocus) {
			this.toolbarIconsTabIndex = (hasFocus) ? 0 : -1;
		},

		reset(newContent) {
			this.content = newContent || '';
			this.$refs.wysiwyg.reset();
			this.$refs.source.value = '';
			this.updatePreview();
		},

		sourceChanged() {
			this.$refs.wysiwyg.setContent(this.content);
			this.eitherEditorContentChanged();
			this.updatePreview();
		},

		toolbarClicked() {
			this.sourceChanged();
		},

		updateContent(data) {
			const thisEditor = this;

			switch (data.insertionType) {
				case 'inline':
					thisEditor.insertAsInline(thisEditor.content, data.command, thisEditor.textArea.selectionStart, thisEditor.textArea.selectionEnd);
					break;
				default:
					const newContent = this.buildCommand(data.command, data.options);
					thisEditor.insertAsCharacter(thisEditor.content, newContent, thisEditor.textArea.selectionStart, thisEditor.textArea.selectionEnd);
			}
			this.eitherEditorContentChanged();
			this.sourceChanged();
		},

		updatePreview() {
			this.$refs.preview.render(this.content);
		},

		wysiwygChanged()
		{
			this.eitherEditorContentChanged();
			this.updatePreview();
		}
	},

	mounted() {
		const thisTextEditor = this,
			$el = $(this.$el);
		$el.on('focusin', function (e) {
			thisTextEditor.onFocusChange(true);
		});
		$el.on('focusout', function (e) {
			thisTextEditor.onFocusChange();
		});
		thisTextEditor.textArea = $(thisTextEditor.$el).find('[data-editor-textarea]')[0];
		$(thisTextEditor.$el).foundation();
	}
}