import ExtractEmails from '../../../../../utilities/utility-libraries/local-files/plugins/helpers/extract-emails.js';

export default
{
	data()
	{
		return {
			singleSourceEmail: '',
			multipleSourceEmails: '',
			fileSourceEmails: '',
			parsedFileSourceEmails: '',
			emailsToInvite: [],

			requiredEmailValidator: null
		};
	},

	props: {
		limit: {
			default: 5,
			type: Number
		}
	},

	methods: {
		addSingleEmail(event)
		{
			event.preventDefault();
			const errorMessage = this.requiredEmailValidator.check(this.singleSourceEmail);
			if (errorMessage)
			{
				// this.showErrorMessage(this.singleSourceEmail, errorMessage);
				this.$toasted.show(errorMessage);
			}
			else
			{
				this.emailsToInvite.push(this.singleSourceEmail);
				this.propagateUpdatedInvitationList();
				this.$refs.singleEmail.reset();
			}
		},

		addMultipleEmails(event)
		{
			event.preventDefault();

			const emailParser = new ExtractEmails(this.$utilities.validators.requiredEmailValidator()),
				emails = emailParser.parse(this.multipleSourceEmails);

			emails.validated.forEach((email) => {
				if (-1 === this.emailsToInvite.indexOf(email))
				{
					this.emailsToInvite.push(email);
				}
			});

			this.propagateUpdatedInvitationList();
			if (emails.failed.length)
			{
				this.$toasted.show(this.$i18n.replace('messages.parsedEmails.failed', {number: emails.failed.length}));
				this.$refs.multipleEmails.reset(emails.failed.join('\n'));
			}
			else
			{
				this.$refs.multipleEmails.reset();
			}
		},

		filesLoaded(fileObjects)
		{
			const options = {plugins: {emails: true}};
			for (let lpFiles = 0; lpFiles < fileObjects.length; lpFiles++)
			{
				this.$utilities.localFile.load(fileObjects.item(lpFiles), options).then((parsedEmails) => {
					parsedEmails.validated.forEach((email) => {
						if (-1 === this.emailsToInvite.indexOf(email))
						{
							this.emailsToInvite.push(email);
						}
					});

					parsedEmails.failed.forEach((email) => {
						this.$toasted.show(email, {type: 'error'});
					});

					this.$refs.emailsFromFile.reset();
				});
			}
		},

		propagateUpdatedInvitationList()
		{
			this.$emit('input', this.emailsToInvite);
		}
	},

	mounted()
	{
		this.requiredEmailValidator = this.$utilities.validators.requiredEmailValidator();
		$(this.$el).foundation();
	}
}