import settings from '../../../../../../settings.js';

const HslaColors = function ()
{
	const thisHsla = this,
		colorConfig = settings.generatedIcons.colors,

		internalConversions = {
			/**
			 * 
			 * @param {object} hslObject
			 * @returns string
			 */
			hslToString: function (hslObject)
			{
				return 'hsl(' + [
					hslObject.h + 'deg',
					hslObject.s + '%',
					hslObject.l + '%'
				].join(',') + ')';
			},

			/**
			 * 
			 * @param {object} hslaObject
			 * @returns string
			 */
			 hslaToString: function (hslaObject)
			 {
				 return 'hsl(' + [
					hslaObject.h + 'deg',
					hslaObject.s + '%',
					hslaObject.l + '%',
					hslaObject.a / 100.0
				 ].join(',') + ')';
			 }
		},
		
		/**
		 * @param {object} minMaxConfig
		 * @returns integer
		 * generates a random integer within the given min and max values
		 */
		generateRandomPercentColorComponent = function (minMaxConfig)
		{
			const max = minMaxConfig.max,
				min = minMaxConfig.min;
			return parseInt((Math.random() * (max - min)) + min);
		};

	/**
	 * @param {object} hslaColor
	 * @returns string
	 */
	thisHsla.colorObjectToString = function (hslaColor)
	{
		return ('a' in hslaColor) ? internalConversions.hslaToString(hslaColor) : internalConversions.hslToString(hslaColor);
	}

	thisHsla.generators = {

		backgroundColor: function (hslPrimaryColor)
		{
			const config = colorConfig.background;
			return {
				h: thisHsla.varyColorComponent(hslPrimaryColor.h, config.hue.variation),
				s: thisHsla.varyColorComponent(hslPrimaryColor.s, config.saturation.variation, config.saturation),
				l: thisHsla.varyColorComponent(hslPrimaryColor.l, config.lightness.variation, config.lightness)
			}
		},


		primaryColor: function (favoriteColorName)
		{
			const config = colorConfig.primary;
			const preferredColor = colorConfig.favorites.colors[favoriteColorName || 'blue'];
			return {
				h: (parseInt(360 * Math.random() * Math.random()) + preferredColor.h) % 360,
				s: generateRandomPercentColorComponent(config.saturation),
				l: generateRandomPercentColorComponent(config.lightness)
			}
		},


		relatedColor: function (hslPrimaryColor)
		{
			const config = colorConfig.related;
			return {
				h: thisHsla.varyColorComponent(hslPrimaryColor.h, config.hue.variation),
				s: thisHsla.varyColorComponent(hslPrimaryColor.s, config.saturation.variation, config.saturation),
				l: thisHsla.varyColorComponent(hslPrimaryColor.l, config.lightness.variation, config.lightness),
				a: parseInt(Math.random() * (config.alpha.max - config.alpha.min) + config.alpha.min)
			}
		}
	}

	/**
	 * @param {integer} primaryValue
	 * @param {integer} variation
	 * @param {object|undefined} minMaxConfig
	 * @returns integer
	 * generated random value centered around primaryValue within +/- variation / 2
	 * if minMaxConfig is truthy clamps the result within the min to max range
	 */
	 thisHsla.varyColorComponent = function (primaryValue, variation, minMaxConfig)
	{
		let variedValue = parseInt(primaryValue + variation * (Math.random() - 0.5));
		if (minMaxConfig)
		{
			variedValue = (variedValue > minMaxConfig.max) ? minMaxConfig.max : variedValue;
			variedValue = (variedValue < minMaxConfig.min) ? minMaxConfig.min : variedValue;
		}
		return variedValue;
	}
}

export default HslaColors