import settings from '../../../../../../settings.js';
import GeneratedIconUtilities from '../generated-icon-utilities.js';

const LayerGenerator = function (hslaPrimaryColor)
{
	const thisLayer = this,
		iconConfig = settings.generatedIcons,
		generatorUtilities = new GeneratedIconUtilities,
		availableShapes = generatorUtilities.getAvailableShapes(),

		/**
		 * @returns boolean
		 */
		coinToss = function ()
		{
			return Math.random() > 0.5;
		},

		randomPosition = function (){
			return 100.0 * Math.random();
		},

		shapeGenerators = {
			circle: function (config){
				const degToRad = function (angle) {
					return angle / 180.0 * Math.PI;
				}
				const angleStart = 360.0 * Math.random();
				return {
					x: randomPosition(),
					y: randomPosition(),
					r: generatorUtilities.getRandomConstrainedValue(config.radius.min, config.radius.max),
					angleStart: degToRad(angleStart),
					angleEnd: degToRad((angleStart + 360.0 * Math.random()))
				}
			},

			polygonRandom: function (config){
				const numberVertices = generatorUtilities.getRandomConstrainedValue(config.vertices.min, config.vertices.max, true),
					vertices = [];
				for (let i = 0; i < numberVertices; i++) {
					vertices.push({
						x: randomPosition(),
						y: randomPosition()
					});
				}
				return vertices;
			},

			polygonRegular: function (config){

			},

			rectangle: function (config){
				return {
					x: randomPosition(),
					y: randomPosition(),
					h: generatorUtilities.getRandomConstrainedValue(0, config.height),
					w: generatorUtilities.getRandomConstrainedValue(0, config.width)
				}
			}
		}

	/**
	 * 
	 * @returns object
	 */
	this.generate = function ()
	{
		const transforms = {}
		const styles = {}
		styles.line = {
			color: generatorUtilities.colors.hsla.generators.relatedColor(hslaPrimaryColor),
			widthPercent: generatorUtilities.getRandomConstrainedValue(iconConfig.borders.min, iconConfig.borders.max)
		};
		// if (coinToss()) {
			styles.fill = generatorUtilities.colors.hsla.generators.relatedColor(hslaPrimaryColor);
		// }
		const shape = {}
		const shapeType = availableShapes[parseInt(Math.random() * availableShapes.length)];
		shape.type = shapeType;
		shape.data = shapeGenerators[shapeType](iconConfig.shapes[shapeType]);

		return {
			transforms: transforms,
			styles: styles,
			shape: shape
		}
	}
}

export default LayerGenerator