import GeneratedIconUtilities from '../generated-icon-utilities.js';

const LayerRenderer = function (canvas)
{
	const thisRenderer = this,
		drawingContext = canvas.getContext('2d'),
		generatorUtilities = new GeneratedIconUtilities,


		// centerAndScaleContext = function ()
		// {
		// 	const widthD2 = 0.5 * canvas.width,
		// 		heightD2 = 0.5 * canvas.height;
		// 	drawingContext.translate(widthD2, heightD2);
		// 	drawingContext.scale(100.0 / widthD2, 100.0 / heightD2);
		// },

		coordinates = {
			x: function (xPercent)
			{
				return canvas.width * xPercent / 100.0;
			},

			y: function (yPercent)
			{
				return canvas.height * yPercent / 100.0;
			}
		},

		renderers = {
			circle: function (shapeData) {
				drawingContext.beginPath();
				drawingContext.arc(
					coordinates.x(shapeData.x), coordinates.y(shapeData.y),
					coordinates.x(shapeData.r),
					shapeData.angleStart, shapeData.angleEnd
				);
				drawingContext.stroke();
			},

			polygonRandom: function (shapeData) {
				drawingContext.beginPath();
				drawingContext.moveTo(coordinates.x(shapeData[0].x), coordinates.y(shapeData[0].y));
				for (let i = 1; i < shapeData.length; i++) {
					drawingContext.lineTo(coordinates.x(shapeData[i].x), coordinates.y(shapeData[i].y));
				}
				drawingContext.stroke();
			},

			rectangle: function (shapeData) {
				drawingContext.fillRect(
					coordinates.x(shapeData.x), coordinates.y(shapeData.y),
					coordinates.x(shapeData.w), coordinates.y(shapeData.h)
				);
			}
		},


		resetDrawingContext = function ()
		{
			drawingContext.resetTransform();
			drawingContext.globalAlpha = 1.0;
			drawingContext.globalCompositeOperation = 'source-over';
			drawingContext.lineJoin = 'round';
		},


		setFillStyle = function (style)
		{
			drawingContext.fillStyle = generatorUtilities.colors.hsla.colorObjectToString(style);
		},


		setLineStyle = function (style)
		{
			drawingContext.lineWidth = style.widthPercent;
			drawingContext.strokeStyle = generatorUtilities.colors.hsla.colorObjectToString(style.color);
		};


	thisRenderer.renderLayer = function (layerDetails)
	{
		resetDrawingContext();
		// centerAndScaleContext();
		setLineStyle(layerDetails.styles.line);
		if ('fill' in layerDetails.styles) {
			setFillStyle(layerDetails.styles.fill);
		}
		renderers[layerDetails.shape.type](layerDetails.shape.data);
	}


	thisRenderer.renderBackground = function (hslaBackground)
	{
		resetDrawingContext();
		drawingContext.fillStyle = generatorUtilities.colors.hsla.colorObjectToString(hslaBackground);
		drawingContext.fillRect(0, 0, canvas.width, canvas.height);
	}
}

export default LayerRenderer